/* (C) 2013 Bogdan Cristian Tataroiu */

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

#define MAXprodNK 1000000
#define MAXWD 1000

struct TrieNode {
    int bst, cnt;
    struct TrieNode *nxt[3];
};

typedef struct TrieNode TrieNode;

int N, K, W, D;
unsigned char cur[MAXprodNK];

TrieNode *root;
int trieNodeCount;

inline TrieNode *newTrieNode() {
    TrieNode *node = (TrieNode *)malloc(sizeof(TrieNode));
    node->bst = 0;
    node->cnt = 0;
    node->nxt[0] = NULL;
    node->nxt[1] = NULL;
    node->nxt[2] = NULL;

    trieNodeCount += 1;
    return node;
}

inline int childCnt(TrieNode *node, int child) {
    if (node->nxt[child] == NULL) {
        return 0;
    }
    return node->nxt[child]->cnt;
}

inline int childBst(TrieNode *node, int child) {
    if (node->nxt[child] == NULL) {
        return 0;
    }
    return node->nxt[child]->bst;
}

inline int max(int a, int b) {
    if (a > b) {
        return a;
    }
    return b;
}

inline void updateTrie(TrieNode *node, int pos) {
    node->cnt += 1;
    if (node->nxt[cur[pos]] == NULL) {
        node->nxt[cur[pos]] = newTrieNode();
    }
    if (pos < K) {
        updateTrie(node->nxt[cur[pos]], pos + 1);
    }

    if (pos == K) {
        node->bst = D * node->cnt;
    } else {
        node->bst = -0x3f3f3f3f;
        int k;
        for (k = 0; k < 3; k++) {
            node->bst = max(node->bst,
                    childBst(node, k) + W * childCnt(node, (k + 1) % 3));
        }
    }
}

inline void printBest(TrieNode *node, int pos) {
    if (pos == K) {
        return;
    }
    if (node == NULL) {
        putchar('P');
        printBest(NULL, pos + 1);
        return;
    }
    int k;
    for (k = 0; k < 3; k++) {
        if (node->bst == childBst(node, k) + W * childCnt(node, (k + 1) % 3)) {
            if (k == 0) {
                putchar('P');
            } else if (k == 1) {
                putchar('R');
            } else if (k == 2) {
                putchar('S');
            }
            printBest(node->nxt[k], pos + 1);
            return;
        }
    }
}

inline void freeTrie(TrieNode *node) {
    int k;
    for (k = 0; k < 3; k++) {
        if (node->nxt[k] != NULL) {
            freeTrie(node->nxt[k]);
        }
    }
    free(node);
}

int main() {
    assert(freopen("rps.in", "rt", stdin));
    assert(freopen("rps.out", "wt", stdout));
    assert(scanf("%d %d %d %d", &N, &K, &W, &D) == 4);
    assert(1 <= N * K && N * K <= MAXprodNK);
    assert(-MAXWD <= W && W <= MAXWD);
    assert(-MAXWD <= D && D <= MAXWD);

    trieNodeCount = 0;
    root = newTrieNode();

    int i, j;
    for (i = 0; i < N; i++) {
        for (j = 0; j < K; j++) {
            assert(scanf(" %c", cur + j) == 1);
            assert(cur[j] == 'R' || cur[j] == 'P' || cur[j] == 'S');
            if (cur[j] == 'P') {
                cur[j] = 0;
            } else if (cur[j] == 'R') {
                cur[j] = 1;
            } else if (cur[j] == 'S') {
                cur[j] = 2;
            }
        }

        updateTrie(root, 0);
        printBest(root, 0);
        printf("\n");
    }

    // Don't free up memory
    // freeTrie(root);
    return 0;
}
