// Mugurel Ionut Andreica

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <map>
#include <string>

using namespace std;

#define FILEIN "spnzr.in"

#define LOCAL 0
#define LOCAL_RAND_SEED 2345
#define RANDOM_DICT 0
#define RANDOM_WORDS 0
#define DEBUG 0
#define USE_MASK_ARRAY_FOR_EVAL

// 1 - Use bit masks for separating classes.
// 2 - Use character frequency for separating classes.
// 3 - Simply select the most frequent character.
// 4 - Like 3, but if the most frequent character occurs in all the words try to avoid it.
// 5 - Random
#define EVAL_STRATEGY 1

#define NMAX 100111
#define LMAX 21
#define ALPHA_SIZE 26

map<string, int> dict;
char w[NMAX][LMAX + 5];
int mask[NMAX][ALPHA_SIZE];
int len[NMAX];
int N, T;

void ReadInput() {
	int i, j, L;

	if (LOCAL) {
		T = 100;
		srand(LOCAL_RAND_SEED);
	
		if (RANDOM_DICT && RANDOM_WORDS) {
			// Generate the dictionary randomly.
			N = 100000;
			for (i = 0; i < N; i++) {
				L = 5 + (rand() % (LMAX - 5 + 1));
				for (j = 0; j < L; j++)
					w[i][j] = 'a' + (rand() % 26);
				w[i][L] = 0;
			}
		} else {
			freopen(FILEIN, "r", stdin);
			scanf("%d", &N);
			for (i = 0; i < N; i++) {
				scanf("%s", w[i]);
				dict[w[i]] = i;
			}
			if (!RANDOM_WORDS)
				scanf("%d", &T);
		}
	} else {
		scanf("%d", &N);
		for (i = 0; i < N; i++)
			scanf("%s", w[i]);
		scanf("%d", &T);
	}

	for (i = 0; i < N; i++)
		len[i] = strlen(w[i]);
}

void ComputeMasks() {
	int i, j;

	for (i = 0; i < N; i++) {
		for (j = 0; j < ALPHA_SIZE; j++)
			mask[i][j] = 0;
		for (j = 0; j < len[i]; j++)
			mask[i][w[i][j] - 'a'] |= (1 << j);
	}
}

int hidden_word_idx;
char hidden_word[LMAX + 5];

int cand[2][NMAX], ncand[2], c, a;

#ifdef USE_MASK_ARRAY_FOR_EVAL
int cnt[1 << LMAX], touched[1 << LMAX], touch_idx = 0;
#else
map<int, int> cnt;
map<int, int>::iterator it;
#endif

#define INF_SCORE 1000000000

int EvalQuestion(int ch) {
	int i, widx, ccnt, score = 0;

	if (EVAL_STRATEGY == 1) {
		// Use bit masks.

		#ifdef USE_MASK_ARRAY_FOR_EVAL
		touch_idx++;
		#else
		cnt.clear();
		#endif

		for (i = 0; i < ncand[c]; i++) {
			widx = cand[c][i];

			//fprintf(stderr, "mask=%d\n", mask[widx][ch]);
			
			#ifdef USE_MASK_ARRAY_FOR_EVAL
			if (touched[mask[widx][ch]] != touch_idx) {
				touched[mask[widx][ch]] = touch_idx;
				cnt[mask[widx][ch]] = 0;
			}
			ccnt = cnt[mask[widx][ch]];
			#else
			it = cnt.find(mask[widx][ch]);
			if (it == cnt.end())
				ccnt = 0;
			else
				ccnt = it->second;
			#endif
			
			ccnt++;
			if (ccnt > score)
				score = ccnt;

			cnt[mask[widx][ch]] = ccnt; 
		}
	} else if (EVAL_STRATEGY == 2) {
		for (i = ccnt = 0; i < ncand[c]; i++) {
			widx = cand[c][i];
			if (mask[widx][ch] > 0)
				ccnt++;
		}
	
		if (ccnt == 0)
			score = INF_SCORE;
		else
			score = max(ccnt, ncand[c] - ccnt);
	} else if (EVAL_STRATEGY == 3) {
		for (i = ccnt = 0; i < ncand[c]; i++) {
			widx = cand[c][i];
			if (mask[widx][ch] > 0)
				ccnt++;
		}

		score = ncand[c] - ccnt;		
	} else if (EVAL_STRATEGY == 4) {
		for (i = ccnt = 0; i < ncand[c]; i++) {
			widx = cand[c][i];
			if (mask[widx][ch] > 0)
				ccnt++;
		}

		if (ccnt == 0)
			score = INF_SCORE;
		else if (ccnt == ncand[c])
			score = ccnt;
		else
			score = ncand[c] - ccnt;
	} else if (EVAL_STRATEGY == 5) {
		for (i = ccnt = 0; i < ncand[c]; i++) {
			widx = cand[c][i];
			if (mask[widx][ch] > 0)
				ccnt++;
		}

		if (ccnt == 0)
			score = INF_SCORE;
		else
			score = rand() % 1000;
	} else {
		fprintf(stderr, "Unimplemented evaluation strategy!\n");
		exit(1);
	}

	return score;
}

int NUM_QUESTIONS;

void GuessWord(int L) {
	int i, j, ch, ch_score, best_score, best_ch, cmask, ans, nr, ok;

	if (DEBUG)
		fprintf(stderr, "Guessing a word of length L=%d\n", L);

	NUM_QUESTIONS = 0;
	
	c = 0;
	ncand[c] = 0;
	for (i = 0; i < N; i++)
		if (len[i] == L) {
			cand[c][ncand[c]] = i;
			ncand[c]++;
		}

	cmask = 0;
	while (ncand[c] > 1) {
		if (DEBUG)
			fprintf(stderr, "Number of candidates=%d\n", ncand[c]);

		// Find the best question to ask next.
		for (best_score = INF_SCORE, best_ch = ch = 0; ch < ALPHA_SIZE; ch++)
			if ((cmask & (1 << ch)) == 0) {
				ch_score = EvalQuestion(ch);
				if (ch_score < best_score) {
					best_score = ch_score;
					best_ch = ch;
				}
			}
	
		if (best_score == INF_SCORE) {
			fprintf(stderr, "Cannot differentiate between the candidates!\n");

			if (DEBUG) {
				for (i = 0; i < ncand[c]; i++)
					fprintf(stderr, "%d: %s\n", i + 1, w[cand[c][i]]);
			}

			exit(1);
		}

		// Ask the question.
		cmask |= (1 << best_ch);

		NUM_QUESTIONS++;
		if (DEBUG)
			fprintf(stderr, "Question #%d: %c (score=%d)\n", NUM_QUESTIONS, best_ch + 'a', best_score);
		
		if (LOCAL) {
			ans = mask[hidden_word_idx][best_ch];
		} else {
			printf("0 %c\n", 'a' + best_ch);
			fflush(stdout);
			
			ans = 0;
			scanf("%d", &nr);
			for (i = 0; i < nr; i++) {
				scanf("%d", &j);
				ans |= (1 << j);
			}
		}

		// Filter the words.
		a = c;
		c = 1 - c;			
		ncand[c] = 0;

		for (i = 0; i < ncand[a]; i++) {
			j = cand[a][i];
			if (mask[j][best_ch] == ans) {
				cand[c][ncand[c]] = j;
				ncand[c]++;
			}
		}
	}

	if (ncand[c] != 1) {
		fprintf(stderr, "Bad word length or identical words exist in the dictionary!\n");
		exit(1);
	}

	printf("1 %s\n", w[cand[c][0]]);
	fflush(stdout);
	
	if (!LOCAL) {
		scanf("%d", &ok);
		if (!ok) {
			fprintf(stderr, "The guessed word is incorrect ?!\n");
			exit(1);
		}
	}
}

int myrand() {
	return (rand() % 30000) * (rand() % 30000);
}

void GuessWords() {
	int L, TOTAL_NUM_QUESTIONS = 0;

	for (int widx = 0; widx < T; widx++) {
		if (LOCAL) {
			if (RANDOM_WORDS) {
				hidden_word_idx = myrand() % N;
				if (DEBUG)
					fprintf(stderr, "Random hidden word: %s (idx=%d)\n", w[hidden_word_idx], hidden_word_idx);
			} else {
				scanf("%s", hidden_word);
				hidden_word_idx = dict[hidden_word];
				if (DEBUG)
					fprintf(stderr, "Hidden word: %s (%s idx=%d)\n", hidden_word, w[hidden_word_idx], hidden_word_idx);
			}

			L = len[hidden_word_idx];
		} else {
			if (DEBUG)
				fprintf(stderr, "Reading the length of the next hidden word...\n");
			scanf("%d", &L);
		}

		GuessWord(L);
		fprintf(stderr, "Word #%d: NUM_QUESTIONS=%d\n", widx + 1, NUM_QUESTIONS);
		TOTAL_NUM_QUESTIONS += NUM_QUESTIONS;
	}

	fprintf(stderr, "TOTAL_NUM_QUESTIONS=%d AVG_NUM_QUESTIONS=%.3lf\n", TOTAL_NUM_QUESTIONS, (double) TOTAL_NUM_QUESTIONS / T);
}

int main() {
	int tstart = clock();

	fprintf(stderr, "Evaluation strategy=%d\n", EVAL_STRATEGY);
	
	ReadInput();
	ComputeMasks();
	GuessWords();

	fprintf(stderr, "Duration=%.3lf sec\n", (double) (clock() - tstart) / CLOCKS_PER_SEC);
	return 0;
}
